float4x4 World;
float4x4 View;
float4x4 Projection;

texture SkyBoxTexture;

sampler SkyBoxSampler = 
sampler_state
{
    Texture = <SkyBoxTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
}; 

void VertexShaderFunction(float3 Position : POSITION0,
                         out float4 SkyPos : POSITION0,
                         out float3 SkyCoord : TEXCOORD0)
{
	float3 worldPosition = mul(Position, World);
    float3 viewPosition = mul(worldPosition, View);
    // Calculate projection, moving all vertices to the far clip plane 
    // (w and z both 1.0)
    SkyPos = mul(float4(viewPosition, 1), Projection).xyww;    

    SkyCoord = Position;
}

float4 PixelShaderFunction(float3 SkyCoord : TEXCOORD0) : COLOR0
{
    return texCUBE(SkyBoxSampler, SkyCoord);
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_1_1 PixelShaderFunction();
		
        // We're drawing the inside of a model
        CullMode = None;  
        // We don't want it to obscure objects with a Z < 1
        ZWriteEnable = false; 
    }
}
