using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace EnvMap
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Matrix world, view, projection;

        Model monkey;
        Effect envMappingShader;

        TextureCube envMapTex;

        Vector3 cameraPos = new Vector3(1.0f, 1.0f, -4.0f);
        Vector3 lightPosition;

        float yaw = 0, pitch = 0;

        SkyBox skyBox;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            skyBox = new SkyBox(this);
            Components.Add(skyBox);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            monkey = Content.Load<Model>("Suzanne");
            envMappingShader = Content.Load<Effect>("EnvMapping");
            envMapTex = Content.Load<TextureCube>("envmap");

            world = Matrix.Identity;
            view = Matrix.CreateLookAt(cameraPos, Vector3.Zero, Vector3.Up);
            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 0.1f, 100);

            skyBox.View = view;
            skyBox.Projection = projection;
            skyBox.SkyBoxTexture = envMapTex;

            envMappingShader.Parameters["View"].SetValue(view);
            envMappingShader.Parameters["Projection"].SetValue(projection);

            envMappingShader.Parameters["cameraPosition"].SetValue(cameraPos);
            envMappingShader.Parameters["specularPower"].SetValue(50);
            envMappingShader.Parameters["specularIntensity"].SetValue(1);

            lightPosition = new Vector3(-2, 2, -2);
            envMappingShader.Parameters["lightPosition"].SetValue(lightPosition);
            SetLightColors(envMappingShader);

            ChangeEffect(monkey, envMappingShader);

            //--
            envMappingShader.Parameters["EnvironmentTexture"].SetValue(envMapTex);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbs = Keyboard.GetState();
            // Allows the game to exit
            if (kbs.IsKeyDown(Keys.Escape))
                this.Exit();

            if (kbs.IsKeyDown(Keys.Up))
                pitch += 0.01f;
            else if (kbs.IsKeyDown(Keys.Down))
                pitch -= 0.01f;
            if (kbs.IsKeyDown(Keys.Left))
                yaw -= 0.01f;
            else if (kbs.IsKeyDown(Keys.Right))
                yaw += 0.01f;

            world = Matrix.CreateFromYawPitchRoll(yaw, pitch, 0);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            DrawModel(monkey, world);

            base.Draw(gameTime);
        }

        private void DrawModel(Model model, Matrix world)
        {
            Matrix[] transforms = new Matrix[model.Bones.Count];
            model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                    effect.Parameters["World"].SetValue(transforms[mesh.ParentBone.Index] * world);
                }
                mesh.Draw();
            }
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }

        private void SetLightColors(Effect effect)
        {
            effect.Parameters["ambientLightColor"].SetValue(new Vector4(0.3f, 0.3f, 0.3f, 1));
            effect.Parameters["diffuseLightColor"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
            effect.Parameters["specularLightColor"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
        }
    }
}
