using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace EnvMap
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class SkyBox : Microsoft.Xna.Framework.DrawableGameComponent
    {
        Matrix view, projection;

        public Matrix View
        {
            get { return view; }
            set { view = value; skyBoxEffect.Parameters["View"].SetValue(view); }
        }

        public Matrix Projection
        {
            get { return projection; }
            set { projection = value; skyBoxEffect.Parameters["Projection"].SetValue(Projection); }
        }

        Model skyBoxModel;
        Effect skyBoxEffect;

        TextureCube skyBoxTex;

        public TextureCube SkyBoxTexture { 
            get { return skyBoxTex; }
            set { skyBoxTex = value; skyBoxEffect.Parameters["SkyBoxTexture"].SetValue(skyBoxTex); }
        }

        public SkyBox(Game game)
            : base(game)
        {
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            skyBoxModel = Game.Content.Load<Model>("SphereHighPoly");
            skyBoxEffect = Game.Content.Load<Effect>("SkyBox");

            ChangeEffect(skyBoxModel, skyBoxEffect);

            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }
        
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            Matrix[] transforms = new Matrix[skyBoxModel.Bones.Count];
            skyBoxModel.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in skyBoxModel.Meshes)
            {
                foreach (Effect effect in mesh.Effects)
                {
                    effect.Parameters["World"].SetValue(transforms[mesh.ParentBone.Index]);
                }
                mesh.Draw();
            }
            GraphicsDevice.RenderState.CullMode = CullMode.CullCounterClockwiseFace;
            GraphicsDevice.RenderState.DepthBufferWriteEnable = true;
            base.Draw(gameTime);
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }
    }
}